#ifndef EXTENDEDLOGVIEWER_EXTENDEDLOGVIEWER_HPP
#define EXTENDEDLOGVIEWER_EXTENDEDLOGVIEWER_HPP

#include <QIcon>
#include <QMainWindow>
#include <QString>
#include <QVector>

#include <ExtendedLog/LogReader.hpp>

#include "ExtendedLogViewer_global.h"

namespace extLog = ExtendedLog;

namespace ExtendedLogViewer
{
	namespace Ui
	{
		class ExtendedLogViewer;
	}

	/**
	 * @brief The ExtendedLogViewer class
	 */
	class EXTENDEDLOGVIEWER_EXPORT ExtendedLogViewer : public QMainWindow
	{
			Q_OBJECT

		public:

			/**
			 * @brief Create a new Olog viewer window
			 * @param reader The extended log reader to use
			 * @param icon The icon to use for this window
			 * @param appName The name of the application this windows was started from
			 * @param parent The \p QWidget that solves as parent for this one
			 */
			explicit ExtendedLogViewer(
					ExtendedLog::LogReader *reader,
					QIcon icon,
					QString appName,
					QWidget *parent = nullptr);

			~ExtendedLogViewer();

		protected:
			void showEvent(QShowEvent *event);

		private:
			/**
			 * @brief The user interface
			 */
			Ui::ExtendedLogViewer *ui;

			/**
			 * @brief The log reader to use
			 *
			 * Pointer to an implementation of the extended log reader from <i>ExtendedLog</i>-project
			 */
			extLog::LogReader *logReader;

			/**
			 * @brief All log entries with type information
			 */
			QVector<extLog::LogEntry> logEntriesInfoType;

			/**
			 * @brief All log entries with type warning
			 */
			QVector<extLog::LogEntry> logEntriesWarningType;

			/**
			 * @brief All log entries with type error
			 */
			QVector<extLog::LogEntry> logEntriesErrorType;

			/**
			 * @brief All log entries to display
			 */
			QVector<extLog::LogEntry> selectedLogEntries;

			/**
			 * @brief Private ctor, initialize datastructures and window
			 * @param reader The log reader to use
			 * @param appName The name of the application this windows was started from
			 * @param parent The \p QWidget that solves as parent for this one
			 */
			explicit ExtendedLogViewer(extLog::LogReader *reader, QString appName, QWidget *parent = nullptr);

			/**
			 * @brief Display all selected entries
			 * @param entries The entries to display
			 */
			void displayEntries(QVector<extLog::LogEntry> &entries);

			/**
			 * @brief Get the localized varaint of an entry type
			 * @param type The type to localize.
			 * @return The localized varaint of an entry type
			 */
			QString entryTypeToLocalizedString(extLog::EntryType type);

		private slots:
			/**
			 * @brief Update selection of the entires according to the filter mask
			 */
			void toolBtnToggled();
	};


} // namespace ExtendedLogViewer
#endif // EXTENDEDLOGVIEWER_EXTENDEDLOGVIEWER_HPP
