#include <ExtendedLog/LogReader.hpp>
#include <ExtendedLog/EntryType.hpp>

#include "ExtendedLogViewer.hpp"
#include "ui_ExtendedLogViewer.h"

#include <QMainWindow>
#include <QString>
#include <QStringList>
#include <QTableWidgetItem>
#include <QVector>

namespace ExtendedLogViewer
{

	ExtendedLogViewer::ExtendedLogViewer(extLog::LogReader *reader, QString appName, QWidget *parent) :
		QMainWindow(parent),
		ui(new Ui::ExtendedLogViewer),
		logReader(reader),
		selectedLogEntries()
	{
		ui->setupUi(this);
		this->setWindowTitle(appName + tr(": show log"));
		logEntriesErrorType = reader->getEntriesByType(extLog::EntryType::ERROR);
		logEntriesWarningType = reader->getEntriesByType(extLog::EntryType::WARNING);
		logEntriesInfoType = reader->getEntriesByType(extLog::EntryType::INFORMATION);
	}

	ExtendedLogViewer::ExtendedLogViewer(ExtendedLog::LogReader *reader, QIcon icon, QString appName, QWidget *parent):
		ExtendedLogViewer(reader, appName, parent)
	{
		this->setWindowIcon(icon);
	}

	ExtendedLogViewer::~ExtendedLogViewer()
	{
		delete logReader;
		delete ui;
	}

	void ExtendedLogViewer::showEvent(QShowEvent *event)
	{
		QMainWindow::showEvent(event);
		ui->lblFileName->setText(logReader->filename);
		QVector<extLog::LogEntry> &entries = logReader->getAllEntries();
		displayEntries(entries);

		//QString buttonLabels[] = {tr("Informations"), tr("Warnings"), tr("Errors")};
		QString errorsText = tr("%n Error(s)", "", logEntriesErrorType.count());
		ui->tbtnError->setText(errorsText);

		QString warningsText = tr("%n Warning(s)", "", logEntriesWarningType.count());
		ui->tbtnWarning->setText(warningsText);

		QString infosText = tr("%n Information(s)", "", logEntriesInfoType.count());
		ui->tbtnInfo->setText(infosText);

        QStringList columnHeaders{};
        columnHeaders << tr("Type") << tr("Date and Time") << tr("Message");
        ui->dataGrid->setHorizontalHeaderLabels(columnHeaders);

	}

	void ExtendedLogViewer::toolBtnToggled()
	{
		selectedLogEntries.clear();
		if (ui->tbtnError->isChecked()) { selectedLogEntries.append(logEntriesErrorType); }
		if (ui->tbtnWarning->isChecked()) { selectedLogEntries.append(logEntriesWarningType); }
		if (ui->tbtnInfo->isChecked()) { selectedLogEntries.append(logEntriesInfoType); }
		displayEntries(selectedLogEntries);
	}

	void ExtendedLogViewer::displayEntries(QVector<ExtendedLog::LogEntry> &entries)
	{
		ui->dataGrid->clear();
        ui->dataGrid->setRowCount(entries.count());
		int rowCounter = 0;
		for (extLog::LogEntry item : entries)
		{
            QTableWidgetItem *typeItem = new QTableWidgetItem(entryTypeToLocalizedString(item.getEntryType()));
			ui->dataGrid->setItem(rowCounter, 0, typeItem);

            QTableWidgetItem *dateTimeItem = new QTableWidgetItem(item.getDateTime().toString("dd.MM.yyyy hh:mm:ss"));
			ui->dataGrid->setItem(rowCounter, 1, dateTimeItem);

			QTableWidgetItem *messageItem = new QTableWidgetItem(item.getMessage());
			ui->dataGrid->setItem(rowCounter, 2, messageItem);

			rowCounter++;
		}
	}

	QString ExtendedLogViewer::entryTypeToLocalizedString(ExtendedLog::EntryType type)
	{
	switch (type)
		{
			case ExtendedLog::EntryType::ERROR:
				return tr("Error");
			case ExtendedLog::EntryType::INFORMATION:
				return tr("Information");
			case ExtendedLog::EntryType::UNKNOWN:
				return tr("Unknown");
			case ExtendedLog::EntryType::WARNING:
				return tr("Warning");
		}
		return tr("Unknown");
	}

} // namespace ExtendedLogViewer
